#include<math.h>

float prho(int n, float is, int *ifault) {
/*
    Algorithm AS 89   Appl. Statist. (1975) Vol.24, No. 3, P377.
       
    To evaluate the probability of obtaining a value greater than or
    equal to is, where is=(n**3-n)*(1-r)/6, r=Spearman's rho and n
    must be greater than 1

    n is the number of data pairs in the correlation
    The calculated probability is for one-sided tests.
    For two-sided tests multiply the probability value with 2.

    Auxiliary function required: ALNORM = algorithm AS66
*/

  float  c1=0.2274, c2=0.2531, c3=0.1745, c4=0.0758, c5=0.1033, c6=0.3932;
  float  c7=0.0879, c8=0.0151, c9=0.0072, c10=0.0831, c11=0.0131, c12=0.00046;
  float  rc;
  double b, x, y, u;
  long   i, js, ifr, nfac, m, n1, mt, nn, ise;
  long   l[7];

  extern float alnorm(float x, int upper);

  /* Test admissibility of arguments and initialize */
  rc=1.0;
  *ifault=1;
  if (n<=1) return(rc);
  *ifault=0;
  if (is<=0) return(rc);
  rc=0.0;
  if (is > n*(n*n-1)/3) return(rc);
  js=(long)is;
  if (js!= 2*(js/2)) js=js+1;
  /*  if (n>6) goto mark6; */
  goto mark6;

  /* Exact evaluation of probability */
  nfac=1;
  for (i=1; i<=n; i++) {
    nfac*=i;
    l[i]=i;
  }
mark1:
  rc=1.0/(float)(nfac);
  if (js != n*(n*n-1)/3) return(rc);
  ifr=0;
  for (m=1; m<=nfac; m++) {
    ise=0;
    for (i=1; i<=n; i++) {
      ise+=(i-l[i])*(i-l[i]);
    }
  mark2:
    if (js <= ise) ifr++;
    n1=n;
  mark3:
    mt=l[1];
    nn=n1-1;
    for (i=1; i<=nn; i++) {
      l[i]=l[i+1];
    }
  mark4:
    l[n1]=mt;
    if ((l[n1]!=n1) || (n1==2)) goto mark5;
    n1--;
    if (m!=nfac) goto mark3;
  }
mark5:
  rc=(float)(ifr)/(float)(nfac);
  return(rc);
  /* Evaluation by Edgeworth series expansion */
mark6:
  b=1.0/(double)(n);
  x=(6.0*((double)(js)-1.0)*b/(1.0/(b*b)-1.0)-1.0)*sqrt(1.0/b-1.0);
  y=x*x;
  u=x*b*(c1+b*(c2+c3*b)+y*(-c4+b*(c5+c6*b)-y*b*(c7+c8*b-y*(c9-c10*b+y*b*(c11-c12*y)))));
  /* Call to algorithm AS 66 */
  rc=u/exp(y/2.0)+alnorm(x,1);
  if (rc < 0.0) rc=0.0;
  if (rc > 1.0) rc=1.0;
  return(rc);
}
